/*************************************************************************
*  Program:       PCNNGR.C                                               *
*  Description:   Graphicfunctions for VieNet2 on PCs (MSDOS)            *
*  Version:       1.2                                                    *
*  Last modified: 20.9.93                                                *
*  Author:        LG                                                     *
*  Copyright (C) Austrian Research Institute for Artificial Intelligence *
*                                                                        *
*  Remarks:  runs on EGA/VGA in 16 graylevels                            *
*            and on Hercules in 16 pseudo-graylevels with dithering      *
**************************************************************************/

#define MODULE

#include <conio.h>
#include <dos.h>
#include <graphics.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include "neurodef.h"
#include "neurons.h"
#include "nngraph.h"

#define SOURCENAME "pcnngr"

#define BACK(col) (((col) & 0x00f0) >> 4)
#define FRONT(col) ((col) & 0x000f)

#define mouseInt int86(0x33, &regs, &regs)
#define min(a,b) ((a>b)?b:a)

static old_colorreg[64*3];
static struct palettetype new_palette, old_palette;

/* Interne Funktionen */
void show_shadow(pINT x, pINT y, pINT width, pINT height, pINT status);
void set_palette(void);
void reset_palette(void);

/* Dummyfunctions for compatibility with X- and WIN-Graphics */
BOOL nn_break(void){return 0;};
void check_args(int argc,char** argv) {};
void x_draw_minmax_weights(void) {};
void msg_print(char* str) {};
pINT load_layout(char* fname){}  /* dummy */
void draw_scaletext(REAL min,REAL max, char* title){};
#ifdef __cplusplus
void init_xgraph(int argc, char** argv, int,int,int,int) {};
#else
void init_xgraph(int argc, char** argv) {};
#endif

/* the following are the original definitions from mouse.h */
/* they will be taken to be private here                   */
int    graphDriver;             /* The Graphics device driver           */
int    graphMode;                       /* The Graphics mode value                */
int    errorCode;                       /* Reports any graphics errors  */
union  REGS regs;     /* Used for interrupts                        */
int maxX, maxY;                         /* The maximum resolution of the screen */
int maxColors;                          /* The maximum # of colors available      */

/* first the mouse routines, all private */
void initMouse(void)
{ regs.x.ax = 0; mouseInt;
  regs.x.ax = 1; mouseInt;
}

void showMouse(void)
{ regs.x.ax = 1; mouseInt; }

void hideMouse(void)
{ regs.x.ax = 2; mouseInt; }

void mouseStatus(void)
{ regs.x.ax = 3; mouseInt;
  mouse_button = regs.x.bx;
  mouse_x = regs.x.cx;
  mouse_y = regs.x.dx;
}

void setMousePosition(int mouse_x, int mouse_y)
{ regs.x.ax = 4;
  regs.x.cx = mouse_x;
  regs.x.dx = mouse_y;
  mouseInt;
}

pINT setCol(double x)
{ int col;
        x=min(fabs(x),XMAX);  /* Normierung auf 16 Grauwerte */
        col=15-(int)15.0/XMAX*x;
  setcolor(col);
        setfillstyle(SOLID_FILL,col);
        return(col);
}

void set_linestyle(pINT col,pINT style,pINT width)
{       if (style)      setlinestyle(DOTTED_LINE,0,width);
        else                            setlinestyle(SOLID_LINE,0,width);
        setcolor(col);
}

void draw_line(pINT x, pINT y, pINT x1, pINT y1)
{ line(x,y,x1,y1); }

void draw_circle(pINT x, pINT y, pINT rd)
{       circle(x,y,rd); }

void draw_bar(pINT col, pINT x, pINT y, pINT x1, pINT y1)
{       if (graphDriver!=HERCMONO)
                {setfillstyle(SOLID_FILL,col);
                bar(x,y,x1,y1);}
        else
                {/* Herculesmode - Graustufen durch Dithermatrix */
                register int i,j;
                col++; /* Dithermatrix erlaubt 17 Stufen, davon nur 16 verwenden */
                if (col==1) col=0;
                for (i=x;i<=x1;i++)
                        for (j=y;j<=y1;j++)
                                if (col>dither[i%4][j%4])
                                        putpixel(i,j,15);
                                else
                                        putpixel(i,j,0);
                }
}

/* Lschen eines Rechtecks mit weiss (col=1) oder schwarz (col=0) */
void clear_box(pINT col, pINT x1, pINT y1, pINT x2, pINT y2)
{ setfillstyle(SOLID_FILL,col);
        bar(x1,y1,x2,y2); }

/*ͻ
   Unit mit Aktivierung val anzeigen, negative Units sind durchgestrichen 
  ͼ*/
void draw_unit(REAL val,pINT x,pINT y, pINT ux, pINT uy)
                                                                                                                                                                                /* dummy Parameter */
{ int col;                                      /* Normierung auf 16 Grauwerte */
  col=15-(int)15.0/XMAX*min(fabs(val),XMAX);
  draw_bar(col,x,y,x+ux,y+uy);
  if (graphDriver==HERCMONO){
     setcolor(0);
     rectangle(x,y,x+ux,y+uy);
     }
  if (val<0){ /* neg.Units erhalten umrandeten Strich in der Mitte */
     setcolor(0);
     rectangle(x,y+uy/2-1,x+ux,y+uy/2+1);
     setcolor(15);
     line(x,y+uy/2,x+ux,y+uy/2);
     }
}

/*ͻ
   Hintergrundrahmen fr Layer l bzw. fr Externlayer anzeigen (ext=1)  
  ͼ*/
void draw_rahmen(pINT l1,pINT ext)
{
    layer L=L_Tab[l1];
    char titel[40];
    int x=L.x, y=L.y;
    if (ext) { x+=L.extsep_x;
               y+=L.extsep_y;
               if (L.titel_ext) strcpy(titel,L.titel_ext);
                  else          sprintf(titel,"%s-X",L.titel);
             }
       else  strcpy(titel,L.titel);
    setfillstyle(SOLID_FILL,8);
    bar(x, y, x+L.dx*(L.upr-1)+L.dx+2, y+L.dy*(L.size/L.upr-1)+L.dy+2);
    gprintf(x, y-10, COL_TEXT, titel);
}

/*ͻ
   Warten auf Tastendruck oder Mausklick                                
  ͼ*/
char nnwait(void)
/* returns a character hit, or the character '@',   */
/* indicating a mouse click, whatever happens first */
{ int mx,my,mb;
        mouse_button = 0;
        while(kbhit()==0 && mouse_button==0)
                mouseStatus();
        if (kbhit()!=0)
                return((char) getch());
        else  {
                mx = mouse_x;
                my = mouse_y;
                mb = mouse_button;
                while(mouse_button!=0)
                        mouseStatus();
                mouse_x = mx;
                mouse_y = my;
                mouse_button = mb;
                return('@');
        }
}

/*ͻ
   Grafik einschalten und initialisieren                                
  ͼ*/
void init_graph(void) {
        graphDriver = DETECT;                   /* Request auto-detection       */
			  errorCode = registerbgidriver(EGAVGA_driver);
				if (errorCode < 0)
          {
          printf("Graphics error: %s\n", grapherrormsg(errorCode));
          printf("Press any key to halt:");
          exit(1); /* terminate with an error code */
          }
        initgraph( &graphDriver, &graphMode, "" );
        errorCode = graphresult();              /* Read result of initialization*/
        if ( errorCode != grOk ) {              /* Error occured during init    */
           printf("\nGraphics System Error: %s\n", grapherrormsg( errorCode ) );
           printf("\nPlease make sure that EGAVGA.BGI is in search path!\n");
           exit( 1 );
        }
        maxColors = getmaxcolor() + 1;  /* Read maximum number of colors*/
        maxX = getmaxx();
        maxY = getmaxy();                               /* Read size of screen          */
        button_x_pos = maxX - BUTTON_WIDTH - 5;
        if (graphDriver==HERCMONO)
                setgraphmode(6);                        /* Grafik-Modus fr Maustreiber */
        else
                set_palette();                          /* sonst VGA-Palette setzen */
        initMouse();
        showMouse();
        settextstyle(SANS_SERIF_FONT,HORIZ_DIR,1);
        UWID = DX - 2;
        if (GP_BORDER) gp_border();
        gprintf(VX,VY,COL_TEXT,VERSION);
}

/*ͻ
   Grafik beenden                                                       
  ͼ*/
void close_graph(void)
{ if (graphDriver!=HERCMONO)
	reset_palette();
  closegraph(); }

void cursor_on(void){ showMouse(); }

void cursor_off(void) { hideMouse(); }

/* Dummies for Compatibility to Apollo */
void g_acquire(void) {}
void g_release(void) {}

/*ͻ
   Setzen der Palette und der Color-Register auf 16 Grauwerte        
  ͼ*/
void set_palette(void) {
        int i;
        struct REGPACK reg;            /* Save Palette */
        getpalette(&old_palette);
        reg.r_ax=(0x10<<8) | 0x17;
        reg.r_cx=64;
        reg.r_bx=0;
        reg.r_dx=FP_OFF(old_colorreg);
        reg.r_es=FP_SEG(old_colorreg);
        intr(0x10,&reg);                                /* Save Color-Registers */
        new_palette.size=16;
        new_palette.colors[0] = 0;  /* palette nichtlinear besetzen */
        new_palette.colors[1] = 15;
        new_palette.colors[2] = 25;
        new_palette.colors[3] = 34;
        new_palette.colors[4] = 38;
        new_palette.colors[5] = 41;
        new_palette.colors[6] = 43;
        new_palette.colors[7] = 45;
        new_palette.colors[8] = 47;
        new_palette.colors[9] = 49;
        new_palette.colors[10] = 51;
        new_palette.colors[11] = 53;
        new_palette.colors[12] = 55;
        new_palette.colors[13] = 57;
        new_palette.colors[14] = 60;
        new_palette.colors[15] = 63;

        for (i=0; i<64; i++)
                {
                reg.r_ax=(0x10<<8) | 0x10;
                reg.r_bx=i;
                reg.r_cx=(i<<8) | i;       /* Color-Register */
                reg.r_dx=i<<8;       /* auf Grautreppe setzen */
                intr(0x10,&reg);
                }
        setallpalette(&new_palette);
}
/*ͻ
    Rcksetzen der Palette und der Color-Register                    
  ͼ*/
void reset_palette(void) {
        struct REGPACK reg;
        setallpalette(&old_palette);
        reg.r_ax=0x10*256+0x12;
        reg.r_cx=64;
        reg.r_bx=0;
        reg.r_dx=FP_OFF(old_colorreg);
        reg.r_es=FP_SEG(old_colorreg);
        intr(0x10,&reg);                                /* Reset Color-Registers */
}

/*ͻ
   Textausgabe im Grafikmodus, Format-Argumente wie printf()          
  ͼ*/
int gprintf( pINT xloc, pINT yloc, int col, char *fmt, ... )
{ va_list  argptr;                                                                              /* Argument list pointer               */
  char str[140];                                                                                  /* Buffer to build sting into      */
  int cnt;                                                                                                /* Result of SPRpINTF for return */
  va_start(argptr, fmt);                  /* Initialize va_ functions      */
  cnt = vsprintf(str, fmt, argptr );      /* prints string to buffer       */
  va_end(argptr);                                                                                 /* Close va_ functions                 */
  if (cnt)
     { /* Ausgabebereich lschen mit Hintergrundfarbe (= col>>4) */
     draw_bar(BACK(col),xloc,yloc,xloc+textwidth(str),yloc+textheight(str));
     setcolor(FRONT(col));
     outtextxy(xloc, yloc, str); /* Send string in graphics mode */
     }
  return(cnt);                /* Return the conversion count      */
}

/*ͻ
   Button anzeigen im Zustand stat=EIN/AUS/NEU                          
  ͼ*/
void show_button(pINT id, pINT stat)
{ unsigned x,y,Color;
	int i;
        char *text  = buttons[id].text;
        char hotkey = buttons[id].key;
        pINT width  = buttons[id].width;
        pINT height = buttons[id].height;
        x=buttons[id].x;
        y=buttons[id].y;
        Color=0+16*8;
        cursor_off();
        setfillstyle(SOLID_FILL,BACK(Color));
        bar(x,y,x+width,y+height);
        setcolor(FRONT(Color));
        switch (stat) {
                case AUS:
                case NEU:
                        outtextxy(x+3,y+3,text);
                        i =(int)(strchr(text,hotkey)-text);
                        if (i>=0)
                                {/* Hotkey im Buttontext unterstreichen */
                                setfillstyle(SOLID_FILL,FRONT(Color));
                                bar(x+3+i*textwidth("A"),y+4+textheight("A"),
                                                        x+2+(i+1)*textwidth("A"),y+4+textheight("A"));
                                }
                        break;
                case EIN:
                        outtextxy(x+6,y+6,buttons[id].text); break;
        } /* end switch */
        if (stat==NEU || stat==AKTIV) stat=AUS;
        if (stat!=PASSIV)
                show_shadow(x,y,width,height,stat);
        buttons[id].status=stat;
        cursor_on();
}
/*ͻ
   Buttonschatten anzeigen im Zustand stat=EIN/AUS                      
        ͼ
*/
void show_shadow(pINT x, pINT y, pINT width, pINT height, pINT status) {
        int i, front;
        if ( status == AUS )
                front = 15;
        else if ( graphDriver == HERCMONO )
                front = 0;
        else
                front = 3;

        /* Top Shadow */
        setwritemode(0);
        setlinestyle(SOLID_LINE, 0, 1);
        setcolor(front);
        for( i = 0; i < 2; i++ ) {
                line(x, y+i, x-i+width, y+i);
                line(x+i, y, x+i, y-i+height);
        }
        if ( status == AUS ) {
                /* Bottom Shadow */
                setcolor(15 - front);
                for ( i = 0; i < 2; i++) {
                        line(x+i, y-i+height, x+width, y-i+height);
                        line(x-i+width, y+i, x-i+width, y+height);
                }
        }
}


#undef SOURCENAME
i++) {
                        line(x+i, y-i+height, x+width, y-i+height);
                        line(x-i+width, y+i, x-i+width, y+height);
                }
        }
}


#undef SOURCENAME
