/*************************************************************************
*  Program:       NNXGRAPH.C                                             *
*  Description:   Graphicfunctions for VieNet2 on X-Windows (SUN)        *
*  Version:       2.03                                                   *
*  Last modified: 1.10.96                                                *
*  Author:        LG                                                     *
*  Copyright (C) Austrian Research Institute for Artificial Intelligence *
**************************************************************************/
/* setenv HELPPATH  auf neurons.info nicht vergessen!! */

 /*
 =============================================================================
 Revision History:

 	1.10.96  LG 1) correcting display of slidervalues with szAnzeige in notify_slider
              2) toggle button status in notify_switch
 =============================================================================
 */


#define MODULE

#include "neurodef.h"
#include "neurons.h"
#include "nngraph.h"

#include <math.h>

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/cms.h>
#include <xview/xv_xrect.h>
#include <xview/scrollbar.h>
#include <xview/termsw.h>

#define SOURCENAME "NNXGRAPH"

#define max(x,y) ((x>y)?(x):(y))
#define min(x,y) ((x<y)?(x):(y))
#define Black 66  /* Color# in colortable */
#define White 0
#define ARG_IS(x) (!strcmp(argv[i],x))

Display     *display;
Window      xid, info_xid, stat_xid;
GC          gc, stat_gc; /* GC used for Xlib drawing */
Frame       frame;
Frame       msg_frame, info_frame, stat_frame;
Textsw      msg_win;
Canvas      info_canvas;
int         black_white_display = False;
int         use_colors = True;

/* Convert sliderposition to variablevalue and vice versa,
   internal resolution is 1/1000 */
#define SliderPosToVar(pos) (((float)(pos)*(buttons[i].val_max-buttons[i].val_min)/1000.0+(float)buttons[i].val_min))
#define SliderVarToPos(var) ((int)(((float)(var)-(float)buttons[i].val_min)*1000.0/((float)(buttons[i].val_max-buttons[i].val_min))))


unsigned long *colors; /* the color table */

#define BACK(col) ((col & 0x00f0) >> 4) /* Aufteilen des Farbwertes in */
#define FRONT(col) (col & 0x000f)       /* Hintergrund und Vordergrund */
                                        /* (col=front+16*back)     */

/* these functions are not used for SUN this time */
void bit_blt(pINT fromx,pINT fromy,pINT sizex,pINT sizey,pINT to_x,pINT to_y){}
void bit_blt_inverted(pINT fromx,pINT fromy,pINT sizex,pINT sizey,pINT tox,pINT toy){}
void box(pINT x, pINT y){}
void clear_box(pINT col, pINT x1, pINT y1, pINT x2, pINT y2){}
void close_graph(void){}
void cursor_on(void){}
void cursor_off(void){}
void draw_bar(pINT col, pINT x, pINT y, pINT x1, pINT y1){}
void draw_circle(pINT x, pINT y, pINT rd){}
void draw_line(pINT x, pINT y, pINT x1, pINT y1){}
void draw_scaletext(REAL min,REAL max, char* title) {}
void g_acquire(void){}
void g_release(void){}
void init_graph() {}
pINT load_layout(char* fname) {}
char nnwait(void) {}
void set_linestyle(pINT col, pINT style, pINT width) {}
void set_draw_op(pINT val){}
void show_shadow(pINT x, pINT y, pINT width, pINT height, pINT stat){}

#ifdef __cplusplus
/* old version */
BOOL nn_break(void) {return nn_break(-1);}
// actual version with step display in statusbar
BOOL nn_break(long nr)
{
  MSG msg;
  char str[20]="";
  BOOL bAbortRequested=FALSE;
  if (nr>=0)
		{
		if (!(nr%AutoSaveWeights))
			{
			sprintf(str,"Saving weights to file %s",ActualNet->WeightFile);
			pprintf(str);
			/*NNStatusMsg(str);*/
			save_weights(ActualNet->WeightFile);
			}
		else
			{
			sprintf(str,"Step: %d",nr);
			pprintf(str);
			/*NNStatusMsg(str);*/
			}
		}
  return bAbortRequested;
}

#else
BOOL nn_break(void) {return 0;}
#endif


void rectangle(pINT x, pINT y, pINT x1, pINT y1)
{  XSetForeground(display, gc, colors[Black]);
   XDrawRectangle(display, xid, gc, x, y, x1-x+1, y1-y+1);
}

void draw_unit(float val, pINT x, pINT y, pINT ux, pINT uy)
{
  int h6=(int)(uy/6)-1, negative=0, to_big=0;
  if (val>1.0)  { val = 1.0; to_big=1; }  else
  if (val<-1.0) { val = -1.0; to_big=1; }
  if (val<0.0 && !use_colors) { val *= -1.0; negative = 1; }
  XSetForeground(display, gc, colors[Black]);
  XDrawRectangle(display, xid, gc, x, y, ux, uy);
  XSetForeground(display,gc,colors[33+(int)(32.0*val)]);
  XFillRectangle(display, xid, gc, x+1, y+1, ux-1, uy-1);
  /* Nur fuer Monochrom-Schirme */
  if(negative)
    {
    XSetForeground(display,gc,colors[White]);
    XFillRectangle(display,xid,gc,x+1,y-h6+1,ux-2,h6<<1);
    XSetForeground(display,gc,colors[Black]);
    XDrawRectangle(display,xid,gc,x,y-h6+1,ux,h6<<1);
    XDrawRectangle(display,xid,gc,x+1,y+1,ux-2,uy-2);
    }
  if (to_big)
    {
    XSetForeground(display,gc,colors[Black]);
    XDrawLine(display,xid,gc,x,y,x+ux,y+uy);
    XDrawLine(display,xid,gc,x+ux,y,x,y+uy);
    }
}

void draw_rahmen(pINT l1, pINT ext)
{
   layer L=L_Tab[l1];
   char titel[40];
   int x=L.x, y=L.y;
   if (ext) { x+=L.extsep_x;
              y+=L.extsep_y;
              if (L.titel_ext) strcpy(titel,L.titel_ext);
                 else          sprintf(titel,"%s-X",L.titel);
            }
   else  strcpy(titel,L.titel);
   gprintf(x,y-14,COL_TEXT,titel);
}

int gprintf(pINT x, pINT y, int col, char *fmt, ...)
{ va_list args;
  int cnt;
  char str[140];
  va_start(args,fmt);
  cnt=vsprintf(str,fmt,args);
  XSetForeground(display, gc, colors[White]);
  XFillRectangle(display, xid, gc, x, y, strlen(str)*8, 12);
  XSetForeground(display, gc, colors[Black]);
  XDrawString(display, xid, gc, x, y+9, str, strlen(str));
  va_end(args);
  return(cnt); /* Return the conversion count  */
}

/* Wird von draw_minmax_weights() aufgerufen */
void x_draw_minmax_weights()
{ int i;
  XClearWindow(display,info_xid);
  XSetForeground(display, gc, colors[Black]);
  for (i=0;i<5;i++)
    XDrawString(display, info_xid, gc, 5, 10+15*i, info_msg[i], strlen(info_msg[i]));
  xv_set(info_frame,XV_SHOW,TRUE,NULL);
}

void show_button(pINT id, pINT stat)
{ if (stat==NEU || stat==AKTIV) stat=AUS;
    buttons[id].status=stat;
}


BOOL UnitClicked(event)
Event *event;
{ pINT l1,u1;
  layer *L;
  for(ALL_LAYERS(l1))
    for(L_ALL(l1,u1))
      {
      L=&L_Tab[l1];
      if ( (event_x(event)-PX(l1,u1)>=0) && (event_x(event)-PX(l1,u1) <= L->ux) 
	&& (event_y(event)-PY(l1,u1)>=0) && (event_y(event)-PY(l1,u1) <= L->uy) )
	  {
          layer_clicked=l1; unit_clicked=u1;
          return 1;
          }
      }
  return 0;
}


void event_proc(win, event, arg)
Xv_Window win;
Event *event;
Notify_arg arg;
{
  pINT found,i,l1,u1;
  layer *L;
  if (event_is_ascii(event))
      {
      /* Check for Hotkeys */
      found = 0; i = 0;
      while(i<buttons_used && !found)
         if(buttons[i].key==event_action(event)) found = 1; else i++;
      if (found) do_action(i);
      }
  else
   switch (event_action(event))
   {
   case ACTION_HELP:
        printf("Sorry, no help available.\n");
        break;
   case ACTION_SELECT:
   case ACTION_MENU:
   case ACTION_ADJUST:
        mouse_button = 0;
        if (event_is_down(event))
          {i = 0; found = 0; unit_clicked = 0;
          mouse_x = event_x(event);
          mouse_y = event_y(event);
          switch (event_action(event))
           {
           case ACTION_SELECT: mouse_button = 1; break;
           case ACTION_ADJUST: mouse_button = 3; break;
           case ACTION_MENU:   mouse_button = 2; break;
           }

          if(UnitClicked(event))
            (*action_when_unit_clicked)(layer_clicked,unit_clicked);
          else
            {/* Click on Background - redraw layers */
            draw_layers();
		        sprintf(info_msg[0],"Overview:");
		        sprintf(info_msg[1],"%ld units in %d layers",units_used,L_anz);
		        sprintf(info_msg[2],"%ld weights",weights_used);
		        sprintf(info_msg[3],"");
		        sprintf(info_msg[4],"");
            x_draw_minmax_weights();
            }
          }  /* end if */
        break;
   }
}

/* Textausgabe im Message-Window mit automatischem Scrolling
   wird von gof_printf() aufgerufen */
void msg_print(str)
char* str;
{ textsw_insert(msg_win,str,strlen(str));
  xv_set(msg_frame,XV_SHOW,TRUE,NULL);
}

/* converts value at var pointer of slider# i to float value */
int SliderVarToPosNr(int i)
{ switch (buttons[i].val_typ) {
	case 'f':
		return(SliderVarToPos(*(float*)buttons[i].var));
		break;
	case 's':
		return(SliderVarToPos(*(short*)buttons[i].var));
		break;
	case 'i':
		return(SliderVarToPos(*(int*)buttons[i].var));
		break;
	case 'l':
		return(SliderVarToPos(*(long*)buttons[i].var));
		break;
	}
}


/* Search for corresponding button-structure */
void notify_slider(item,pos,event)
Panel_item item;
int pos;
Event *event;
{ int i=0;
	char szAnzeige[30];
  float value=0.0,oldvalue,res=buttons[i].val_res;
  while (i<buttons_used && buttons[i].id!=(long)item) i++;
  switch (buttons[i].val_typ) {
	case 'f':
		value=*((float*)buttons[i].var);
		break;
	case 'c':
		value=(float)(*((short int*)buttons[i].var));
		break;
	case 'i':
		value=(float)(*((int*)buttons[i].var));
		break;
	case 'l':
		value=(float)(*((long*)buttons[i].var));
		break;
	}
  oldvalue = value;

  value=SliderPosToVar(pos);
  value=max((float)buttons[i].val_min, min((float)buttons[i].val_max,value));
  
  if (value == oldvalue) return;
  
	switch (buttons[i].val_typ) {
	case 'f':
		sprintf(szAnzeige,buttons[i].val_fmt,value);
		*((float*)buttons[i].var)=value;
		break;
	case 'c':
		sprintf(szAnzeige,buttons[i].val_fmt,(short)value);
		*((short int*)buttons[i].var)=(short)value;
		break;
	case 'i':
		sprintf(szAnzeige,buttons[i].val_fmt,(int)value);
		*((int*)buttons[i].var)=(int)value;
		break;
	case 'l':
		sprintf(szAnzeige,buttons[i].val_fmt,(long)value);
		*((long*)buttons[i].var)=(long)value;
		break;
	}

  button_pressed = i; /* Buttonnr. merken */

  /* to be eliminated if slider display works with value display */
  printf("Value of \"%s\" is %s\n",buttons[i].text,szAnzeige);
  
  if (buttons[i].call) ((buttons[i].call)(value));
    
}

void notify_switch(item,value,event)
Panel_item item;
int value;
Event *event;
{ int i=0;
  while (i<buttons_used && buttons[i].id!=(long)item) i++;
  buttons[i].status=!buttons[i].status;
  /*printf("Value of %s: %s\n",buttons[i].text,(buttons[i].status)?"ON":"OFF");*/
  button_pressed = i;     /* Buttonnr. merken */
  ((buttons[i].call)(value));
}

/* Check for special arguments */
void check_args(argc,argv)
int argc;
char **argv;
{ int i;
  if (argc<2) return;
  for (i=1; i<argc; i++)
    if (argv[i][0]=='-')
      {
      if (ARG_IS("-h"))
    	{
        printf("\n%s - A Neural Network Program",argv[0]);
        printf("\nusing the NN-Toolkit VieNet2.\n");
        printf("\nFurther information will be available later...\n\n");
        exit(0);
        }
      if (ARG_IS("-mono"))
    	use_colors = False;
      }
}

/* define our own pow() because of a bug in GNU-mathlib */
float mypow(float x, float y)
{ return exp(y*log(x)); }


/* INIT XView-Graphics */
void init_xgraph(argc,argv)
int argc;
char *argv[];
{
    static char stipple_bits[] = {0xAA, 0xAA, 0x55, 0x55};
    static Xv_singlecolor cms_colors[67],bg,fg,msg_bg,msg_fg;
    int i,j,button_row,slider_val;
    Panel panel;
    Panel_item button;
    Cms cms;
    Canvas canvas, stat_canvas;

    XFontStruct *font;

    XGCValues   gc_val;
    void        canvas_repaint();
    void        info_canvas_repaint();
    Xv_cmsdata  cms_data;
    int         use_dynamic = FALSE;

    fg.red=250, fg.green=230, fg.blue=20;
    bg.red=180, bg.green=100, bg.blue=20;

    /* Setup Colortable to "visual linear" degrading */
    for (i=1; i<33; i++)
      {
      j=round((3.0*i*mypow(i,0.281)));
      /* Bluetable */
      cms_colors[33-i].red   = 255-j;
      cms_colors[33-i].blue  = (use_colors)? 255 : 255-j;
      cms_colors[33-i].green = 255-j;
      /* Redtable */
      cms_colors[33+i].red   = (use_colors)? 255 : 255-j;
      cms_colors[33+i].blue  = 255-j;
      cms_colors[33+i].green = 255-j;
      }
    cms_colors[0].red  = cms_colors[0].green  = cms_colors[0].blue  = 255;
    cms_colors[33].red = cms_colors[33].green = cms_colors[33].blue = 255;
    cms_colors[66].red = cms_colors[66].green = cms_colors[66].blue = 0;

    /* Create windows */
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    frame = xv_create(0,FRAME,
               FRAME_LABEL,    "VieNet2",
               XV_WIDTH,       700,
               XV_HEIGHT,      400,
               NULL);

    msg_frame = xv_create(frame,FRAME,
                FRAME_LABEL,    "VieNet2 - Messages",
                XV_WIDTH,       600,
                XV_HEIGHT,      200,
            /*  FRAME_BACKGROUND_COLOR, &bg,
                FRAME_FOREGROUND_COLOR, &fg,
                FRAME_INHERIT_COLORS, TRUE, */
                NULL);

    xv_set(msg_frame, XV_SHOW, TRUE, NULL);

    info_frame=xv_create(frame, FRAME,
                XV_WIDTH,       200,
                XV_HEIGHT,      100,
                FRAME_LABEL, "VieNet2 - Info",
                NULL);
/*
                stat_frame=xv_create(frame, FRAME,
                XV_WIDTH,       400,
                XV_HEIGHT,      200,
                FRAME_LABEL, "Statistik",
                NULL);
                xv_set(stat_frame, XV_SHOW, TRUE, NULL);
*/
    cms = xv_create(0, CMS,
                    CMS_SIZE,       67,
                    CMS_TYPE,       use_dynamic? XV_DYNAMIC_CMS : XV_STATIC_CMS,
                  CMS_COLORS,     cms_colors,
                        NULL);

    info_canvas=xv_create(info_frame, CANVAS,
                 CANVAS_REPAINT_PROC, info_canvas_repaint,
               CANVAS_X_PAINT_WINDOW, TRUE,
                             WIN_CMS, cms,
                   CANVAS_AUTO_CLEAR, TRUE,
                                NULL);

    panel = (Panel)xv_create(frame, PANEL,
                      PANEL_LAYOUT, PANEL_VERTICAL,
                  PANEL_ITEM_Y_GAP, 5,
                       WIN_ROW_GAP, 8,
                      XV_HELP_DATA, "neurons:Panel",
                              NULL);

    button_row=0;
    for (i=0;i<buttons_used;i++)
      switch (buttons[i].typ) {

        case BUTTON:
             button=xv_create(panel, PANEL_BUTTON,
                               XV_Y, xv_row(panel,button_row),
                 PANEL_LABEL_STRING, buttons[i].text,
                  PANEL_NOTIFY_PROC, buttons[i].call,
                  PANEL_CLIENT_DATA, frame,
                       XV_HELP_DATA, "neurons:QUIT",
                               NULL);
             button_row++;
             buttons[i].id=button;
             break;

        case SWITCH:
             button=xv_create(panel, PANEL_CHECK_BOX,
                       PANEL_LAYOUT, PANEL_HORIZONTAL,
                               XV_Y, xv_row(panel,button_row),
                 PANEL_LABEL_STRING, buttons[i].text,
               PANEL_CHOICE_STRINGS, " ", NULL,
                  PANEL_NOTIFY_PROC, notify_switch,
                        PANEL_VALUE, 1,
                               NULL);
             button_row++;
             buttons[i].id=button;
             break;

        case TWIN_1:
             button=xv_create(panel, PANEL_BUTTON,
                               XV_Y, xv_row(panel,button_row),
                 PANEL_LABEL_STRING, buttons[i].text,
                  PANEL_NOTIFY_PROC, buttons[i].call,
                  PANEL_CLIENT_DATA, frame,
                       XV_HELP_DATA, "neurons:QUIT",
                               NULL);
             buttons[i].id=button;
             break;

        case TWIN_2:
             button=xv_create(panel, PANEL_BUTTON,
                               XV_X, xv_col(panel,(int)(9.0/7.0*strlen(buttons[i-1].text))+2),
                               XV_Y, xv_row(panel,button_row),
                 PANEL_LABEL_STRING, buttons[i].text,
                  PANEL_NOTIFY_PROC, buttons[i].call,
                  PANEL_CLIENT_DATA, frame,
                       XV_HELP_DATA, "neurons:QUIT",
                               NULL);
             buttons[i].id=button;
             button_row++;
             break;

        case SLIDER:
             button=xv_create(panel, PANEL_SLIDER,
                               XV_Y, xv_row(panel,button_row),
                 PANEL_NOTIFY_LEVEL, PANEL_ALL,
                 PANEL_LABEL_STRING, buttons[i].text,
                        PANEL_VALUE, SliderVarToPosNr(i),
                   PANEL_SHOW_VALUE, FALSE,
                   PANEL_SHOW_RANGE, FALSE, /* dont show range, cause its the virtual 0..1000 one! */
                    PANEL_MIN_VALUE, 0,
                    PANEL_MAX_VALUE, 1000,
                 PANEL_SLIDER_WIDTH, 50,
                        PANEL_TICKS, 1,
             PANEL_SLIDER_END_BOXES, FALSE,
                  PANEL_NOTIFY_PROC, notify_slider,
                               NULL);
             buttons[i].id=button;
             button_row++;
             break;
      }

    window_fit_width(panel);

    canvas = xv_create(frame, CANVAS,
         CANVAS_REPAINT_PROC, canvas_repaint,
       CANVAS_X_PAINT_WINDOW, TRUE,
          WIN_DYNAMIC_VISUAL, use_dynamic,
                     WIN_CMS, cms,
                        NULL);

/* Statistik window not used
    stat_canvas = xv_create(frame, CANVAS,
              CANVAS_REPAINT_PROC, stat_canvas_repaint,
            CANVAS_X_PAINT_WINDOW, TRUE,
               WIN_DYNAMIC_VISUAL, use_dynamic,
                          WIN_CMS, cms,
                             NULL);
*/

    msg_win = xv_create(msg_frame, TEXTSW,
                  TEXTSW_BROWSING, TRUE,
           WIN_COLLAPSE_EXPOSURES, FALSE,   /* events sofort anzeigen */
                             NULL);

    window_fit(msg_frame);

    xv_set(canvas_paint_window(canvas),
           WIN_CONSUME_EVENTS,
             WIN_ASCII_EVENTS, LOC_DRAG, WIN_MOUSE_BUTTONS, NULL,
               WIN_EVENT_PROC, event_proc,
                   NULL);

    /* Get display and xid */
    display  = (Display *)xv_get(frame, XV_DISPLAY);
    xid      = (XID)xv_get(canvas_paint_window(canvas), XV_XID);
    info_xid = (XID)xv_get(canvas_paint_window(info_canvas), XV_XID);
    /* stat_xid = (XID)xv_get(canvas_paint_window(stat_canvas), XV_XID);*/

    window_fit(frame);

    if (DefaultDepth(display, DefaultScreen(display)) < 2)
         black_white_display = True;


    if (!(font = XLoadQueryFont(display, "fixed"))) {
       puts("cannot load fixed font");
       exit(1);
       }

    /* Create and initialize GC */
    gc_val.font = font->fid;
    gc_val.graphics_exposures=TRUE;
    gc_val.stipple = XCreateBitmapFromData(display, xid, stipple_bits, 16, 2);
    gc = XCreateGC(display, xid, GCFont | GCStipple, &gc_val);

/*
    gc_val.stipple =
        XCreateBitmapFromData(display, stat_xid, stipple_bits, 16, 2);

    stat_gc = XCreateGC(display, stat_xid, GCFont | GCStipple, &gc_val);
*/
    /* get the colormap from the canvas now that
     * the cms has been installed
     */
    colors = (unsigned long *)xv_get(canvas, WIN_X_COLOR_INDICES);
}

/* Wird von wait_for_action() in neurons.c aufgerufen */
char x_wait_for_action(void) { xv_main_loop(frame); }

/*
 * Draws onto the canvas using Xlib drawing functions.
 */
void canvas_repaint(canvas, pw, display, xid, xrects)
Canvas          canvas;
Xv_Window       pw;
Display         *display;
Window          xid;
Xv_xrectlist    *xrects;
{ int i;
  XSetForeground(display, gc, colors[Black]);
  XDrawString(display, xid, gc, 10, 10, "Activation-Values", 17);
  XDrawString(display, xid ,gc, 7, 30, "-1.0",4);
  XDrawString(display, xid ,gc, 202, 30, "0.0",3);
  XDrawString(display, xid ,gc, 400, 30, "1.0",3);
  XDrawLine(display,xid,gc,210,11,210,18);
  for (i=1;i<66;i++)
      {XSetForeground(display,gc,colors[i]);
      XFillRectangle(display,xid,gc,10+i*6,13,6,6);
      if (i<33 && !use_colors)
         {/* for monochrom displays */
         XSetForeground(display,gc,colors[White]);
         XFillRectangle(display,xid,gc,11+i*6,15,4,2);
/*       XSetForeground(display,gc,colors[Black]);
         XDrawRectangle(display,xid,gc,10+i*6,15,6,2);
*/       }
      }
  draw_layers();
}


/*
void stat_canvas_repaint(canvas, pw, display, xid, xrects)
Canvas          canvas;
Xv_Window       pw;
Display         *display;
Window          xid;
Xv_xrectlist    *xrects;
{   int i;
    XSetForeground(display, stat_gc, colors[Black]);
                XFillRectangle(display, xid, stat_gc, 25, 25, 100, 100);
                XSetFunction(display, stat_gc, GXinvert);
                XFillRectangle(display, xid, stat_gc, 50, 50, 50, 50);
                XSetFunction(display, stat_gc, GXcopy);

                XDrawString(display, xid, stat_gc, 280, 145, "Test-Lines", 10);

                XSetForeground(display, gc, colors[Black]);
                XDrawRectangle(display, xid, stat_gc,
                                5, 5, xv_get(pw, XV_WIDTH)-10, xv_get(pw, XV_HEIGHT)-10);
                XDrawRectangle(display, xid, stat_gc,
                                7, 7, xv_get(pw, XV_WIDTH)-14, xv_get(pw, XV_HEIGHT)-14);
        for (i=0;i<4;i++)
                XDrawString(display, xid, stat_gc, 5, 10+10*i, "HEI", 3);

 }
*/


/* Info-Window Repaint Procedure */
void info_canvas_repaint(canvas, pw, display, xid, xrects)
Canvas          canvas;
Xv_Window       pw;
Display         *display;
Window          xid;
Xv_xrectlist    *xrects;
{
  int i;
  XClearWindow(display,xid);
  XSetForeground(display, gc, colors[Black]);
  for (i=0;i<5;i++)
      XDrawString(display, xid, gc, 5, 10+15*i,
                   info_msg[i], strlen(info_msg[i]));
}








