/*************************************************************************
*  Program:       SLIDERS.C                                              *
*  Description:   Exampleapplication for VieNet2 Sliders & Switches      *
*  Last modified: 04.02.94                                               *
*  Author:        LG                                                     *
**************************************************************************/

/* This file shows the usage of make_switch(..), make_slider(..),
   make_slider_e(..) and printf_file(..) with various parameters
   in a simple 2-Layer-Network
*/

/* sourcefilename for automatic errormessages */
#define SOURCENAME "SLIDERS"

/* headerfiles for VieNet2, don't change the sequence! */
#include "neurodef.h"
#include "neurons.h"
#include "nngraph.h"

/* ========define layervariables globally======== */
pINT A,B;  

/* ========define your own global variables======== */
int  cycles =   5; 
long steps  =  50;
long steps2 = 500;
float rate  = 0.05;
long  patnr  = 0;

/* ========define your connect function======== */
void connect_all_new()
{ nnconnect(A,B,FULL,-1.0,1.0,1.0);}

/* ========define your update function(s)======== */
/* use standard-update */
void upd()
{ update(A,A,EXT,SINGLE);
  update(A,B,STANDARD,SINGLE);
  copy(); 
  draw_layers();
}

/* ========define your learn function(s)======== */
/* simple learning of actual pattern */
void learn()
{ pINT i,j,l;
  upd();
  pprintf("Learning");
	for (l=0;l<steps;l++)
    {
		pprintf(".");

    /* Demo fr Logfile */
		pprintf_file("sliders.log","Learnstep: %i\n",l);

		for (L_ALL(A,i))
		 	for (L_ALL(B,j))
				if ((X(A,i) > 0.5) && (X(B,j) > 0.5))
					W(A,B,i,j) += rate;
				else
		 			W(A,B,i,j) -= rate;
		}
	pprintf("\ndone\n");
}


/* this functions are called after each slider change */
void print_steps()
{pprintf("Steps is: %li\n",steps);}

void print_steps2()
{pprintf("Steps2 is: %li\n",steps2);}

void print_cycles()
{pprintf("Cycles is: %i\n",cycles);}

void print_rate()
{pprintf("Rate is: %.2f\n",rate);}

/* choose actual pattern */
void set_pattern()
{ nth_pat(A,patnr);
  upd();
}
 
/* demo for switch-function */
void print_switch()
{pprintf("Switch is %i\n",buttons[button_pressed].status);}


/*====================define your "main" function========================
	nn_init() initializes all layers, connections, buttons and other stuff
  =======================================================================*/
void nn_init(int argc, char** argv)
{
	init_randn();  /* initialize random-generator (for nnconnect(...)) */

	/* -----init layers - they are not displayed yet------------- */
	A = init_layer(10,1,GRID,100,200,"Layer_A");
	B = init_layer(20,3,GRID,40,80,"Layer_B");

	/* --define the position of the external layer for layer A--- */
	init_ext(A,0,50,"external input for A");

	/* load layout (optional), overwrites settings
		 form init_layer() und init_ext() ! */
	load_layout(LayoutFile);

	/* -----initialize all connections and set their weights---------- */
	connect_all_new(); 

	/* -----define all your buttons, switches and sliders------------- */
#ifndef WIN
  /* windowsversion already supports this functions in its menuline */
	make_button("QUIT",shutdown,'Q'); /* dont forget this! */ 
	make_button("NEW WEIGHTS",connect_all_new,'W');
	make_button("Reset",reset_all,'R');
#endif
	make_button("Update",upd,'U');
	make_button("Learn",learn,'L');

	/* define a switch, which calls print_switch() at every change */
	make_switch("Switch",print_switch,'S');

	/* define a slider for 0..20 for integer variable cycles,
		 print_cycles() will be called for every valuechange.
		 If you don't won't a function to be called, use NULL instead */
	make_slider_e("Cycles:",print_cycles,'C',"%3i",&cycles,0,20,1);

	/* same but with default-parameters (type is long!) */
	make_slider("Steps:",print_steps,' ',&steps,0,200L);

	/* define a slider with your own range and resolution,
		 don't forget the long-"L"! */
	make_slider_e("Steps2:",print_steps2,' ',"%li",&steps2,-2000L,100000L,1000L);

	/* slider with float-variable */
	make_slider_e("Rate:",print_rate,' ',"%.2f",&rate,0.01,0.99,0.01);

	/* define patterns explicit */
	make_pattern_e(A,"1100000000","0");
	make_pattern_e(A,"1110000000","1");
	make_pattern_e(A,"0111000000","2");
	make_pattern_e(A,"0011100000","3");
	make_pattern_e(A,"0001110000","4");
	make_pattern_e(A,"0000111000","5");
	make_pattern_e(A,"0000011100","6");
	make_pattern_e(A,"0000001110","7");
	make_pattern_e(A,"0000000111","8");
	make_pattern_e(A,"0000000011","9");
	make_pattern_e(A,"0000000001","last");
	first_pattern(A);  /* init patternchain */

  /* now use a slider to choose actual pattern */
	make_slider("Pattern:",set_pattern,' ',&patnr,0,L_Tab[A].patanz-1);

	/* initialize graphic and show all */
	init_xgraph(argc,argv); 

	pprintf("This is SLIDERS, a VieNet2 Demo Application \n");
	pprintf("VieNet2 is up to version %s!\n\n",VERSION);

	/* not necessary for Windows, but doesn't matter */
	wait_for_action();     /* Messageloop */
}

/*=========================================================================
	Main() with arguments for supporting the standard X parameters.
	Not defined for Windows, because nn_init() will be called directly
	from WinMain() in nnwingr.cpp!
  =========================================================================*/
#ifndef WIN
void main(int argc, char** argv)
{ nn_setup(argc,argv);
  nn_init(argc,argv);}
#endif

