/*************************************************************************
*  Program:       NEURODEF.H                                             *
*  Description:   userdefined parameters for VieNet2                     *
*  Version:       2.03                                                   *
*  Last modified: 07.07.96                                               *
*  Author:        LG                                                     *
*  Copyright (C) Austrian Research Institute for Artificial Intelligence *
**************************************************************************/

/* default-values for same parameters:  
     XXX_DEF defines the defaultvalue for variable XXX.
 */

/*
----------ATTENTION:---------------------------------------------------------
1.) Computer-type must be defined:
  either:
   for Unix in makefile
   for MSDOS with BC++ 3.x in menu Options/Compiler/Code Generation
     set Defines = PC for MSDOS or WIN for Windows respectively.

  or:
   define the type directly after this comments with 
     #define sun4
   or
     #define PC
   or
     #define WIN
   respectively.

2.) for PCs using Borland/Turbo-C:
      Memory Model    = Large
      Nested Comments = Yes
      Always C++ = Yes (compile all sourcefiles with C++)

  i.e. for working directory \bcpp\neuro.win:
   Include-Directories: \bcpp\include;\bcpp\classlib\include;
                          \bcpp\owl\include;\bcpp\neurons
   Lib-Directories:     \bcpp\lib;\bcpp\classlib\lib;\bcpp\owl\lib
*/

/*==================User changeable======================================*/
/* define NNDEBUG for usage of special debugging tools (memory checks) */
/* #define NNDEBUG */

/* define NNLOGALLOC for logging of all alloc/free-accesses in File alloc.log
   (functions nnmalloc(), nncalloc(), nnrealloc(), nnfree() must be used) */
/* #define NNLOGALLOC */

/* use additional controls, nncontrl.cpp must be linked to project! */
/* #define NNCONTROLS */
/*==================End of user changes==================================*/

#ifndef VERSION
#define VERSION "V2.03"
#endif

/* default defs on PCs */
#ifdef __MSDOS__
#define PC
#endif
#ifdef _Windows
#define WIN
#endif

#ifdef  apollo
#define HTYPE "apollo"
#else
#ifdef  PC
#define HTYPE "pc"
#else
#ifdef  sun4
#define HTYPE "sun4"
#else
#ifdef  WIN
#define PC
#define HTYPE "win"
#else
#error You must define the type of yout host!
#endif
#endif
#endif
#endif

#ifdef PC
#ifndef __LARGE__
#error You must use the LARGE memory model!
#endif
#endif

/************************************************************************/
/* Network related: */
#define MAXLAYER      30       /* max. Number of Layers                 */
#define MAX_W_LEVELS   2       /* max. Number of Weight-Levels <!>      */

/* border-values: (see border_x, border_w ...) */
/* borders for activations */
#define XMIN      0.0
#define XMAX      1.0
#define XMIN_TOL  0.0
#define XMAX_TOL  0.2

/* borders for weights */
#define WMIN     -1.0
#define WMAX      1.0
#define WMIN_TOL -0.3
#define WMAX_TOL  0.3

#define ONTHRES_DEF 0.5

/*****************************************************************************/
/* set break after nnerror(), nnalarm(): */
#define ALARMSTOP         FAIL
#define ERRORSTOP         TRUE

/* redirection of outputs for:
   P .. pprintf
   E .. nn_error
   A .. nn_alarm

  mask the outputchanel with:
   _G: graphicoutput
   _O: textoutput
   _F: fileoutput
*/
#define _G    4
#define _O    2
#define _F    1
#define _GO   6
#define _GOF  7
#define _HIDE 0

#ifndef sun4
#ifndef WIN
#define PG              _G
#define PO              _HIDE
#define PF              _HIDE
#define EG              _G
#define EO              _O
#define EF              _HIDE
#define AG              _G
#define AO              _HIDE
#define AF              _HIDE
#endif
#endif

#ifdef sun4
#define PG              _G
#define PO              _HIDE
#define PF              _HIDE
#define EG              _HIDE
#define EO              _O
#define EF              _HIDE
#define AG              _HIDE
#define AO              _O
#define AF              _HIDE
#endif

#ifdef WIN
#define PG              _G
#define PO              _HIDE
#define PF              _HIDE
#define EG              _G
#define EO              _HIDE
#define EF              _HIDE
#define AG              _G
#define AO              _HIDE
#define AF              _HIDE
#endif

/***************************************************************************/
/* Graphic */

/* dimensions: */
#define MIN_WINDOW_X_SIZE 420
#define MIN_WINDOW_Y_SIZE 420

/* Color for textdisplay */
#define COL_TEXT 15

/* distance between units in pixel (defines width of units, too) */
#ifdef PC
#ifndef WIN
/* DOS */
#define DX_DEF 12
#else
/* Windows */
#define DX_DEF 15
#endif
#else
/* X-Windows */
#define DX_DEF 15
#endif
/* max-length of pattern-name */
#define PNAME_SIZE     15

/* max. Number of defined Buttons */
#define MAXBUTTONS     40

/* Button-Dimensions on Screen in Pixels */
#ifdef PC
#ifdef WIN
/* Windows */
#  define BUTTON_WIDTH   150
#  define BUTTON_HEIGHT  22
#else
/* DOS */
#  define BUTTON_WIDTH   136
#  define BUTTON_HEIGHT  19
#endif
#else
/* X-Windows */
#  define BUTTON_WIDTH   80
#  define BUTTON_HEIGHT  20
#endif

/* Distance between two Buttons on Screen */
#ifdef PC 
#  define BUTTON_DIST  3
#else
#  define BUTTON_DIST  10
#endif

/* BUSY/FREE & VERSION  Position */
#ifndef sun4
#define VX      DX*17
#define VY      3
#define BFX     DX*17+50
#define BFY     3
#else
#define VX      DX*17
#define VY      40
#define BFX     DX*17+50
#define BFY     40
#endif                                       

/*****************************************************************************/
/* where to show min/max values for draw_weights_to and draw_weights_from  */

#ifdef PC 
#  define DRAWW_X_DEF  300
#  define DRAWW_Y_DEF  135
#else
#ifdef Apollo 
#  define DRAWW_X_DEF  400
#  define DRAWW_Y_DEF   35
#else
#  define DRAWW_X_DEF  380
#  define DRAWW_Y_DEF   55
#endif
#endif

#define DRAWW_LINE_SIZE  14

/*****************************************************************************/
/* graphic-scroll-area:   */ 

/* position */
#ifdef PC
#  define GP_X_DEF      5
#  define GP_Y_DEF      300
#else
#  define GP_X_DEF      40
#  define GP_Y_DEF      500
#endif

/* dimensions: */
#define GP_LINE_SIZE  14
#define GP_LINES_DEF  8
#ifdef PC
#  define GP_CHAR_SIZE 8                /* pixelwidth of characters for PC with 640x480 */
#  define GP_COLLS_DEF 60
#else
#  define GP_CHAR_SIZE 7                /* pixelwidth of characters for Apollo */
#  define GP_COLLS_DEF 80
#endif

/* automatically draw border? */
#define GP_BORDER         TRUE

/* distance between border and text */
#define GP_BORD           6

/* max length of textstrings in parameters */
#define GP_BUFFERSIZE 500

/* parameters for f(x) = 1/(1+exp(-GAIN*(x+OFFSET))) */
#define GAIN_DEF 2.0
#define OFFSET_DEF 0.0

/* parameters for STANDARD-update xnew = x + RATE*(-x + f(ALPHA*net + LAST*x) */
#define RATE_DEF 1.0  /* Anteil neuer Einfluss */
#define ALPHA_DEF 1.0 /* Faktor fuer net */
#define LAST_DEF 0.0  /* Momentum fuer X() */

/* parameters for IAC-update */
#define DECAY_DEF 0.1
/* number of IAC-updates per call of update() */
#define MAXIAC_DEF 5

#define EPSILON_DEF 0.15

#define AutoSaveWeights_DEF 10000L /* auto save weights every 10000 steps */
/*****************************************************************************/
