/*************************************************************************
*  Program:       NNLAYER.C                                              *
*  Description:   layer specific functions                               *
*  Version:       1.3                                                    *
*  Last modified: 26.6.93                                                *
*  Author:        LG                                                     *
*  Copyright (C) Austrian Research Institute for Artificial Intelligence *
**************************************************************************/

#define MODULE
//#define SOURCENAME "NNLAYER"
#include "neurodef.h"
#include "neurons.h"
#include "nnlayer.h"
#include <math.h>
#ifdef PC 
#include <stdlib.h>
#endif

/* schnelle Rundung auf Integer (nur Modulintern) */
int round_0(double x)
{ if ((x>=0.0 && x-(int)x>=0.5) || (x<0.0 && x-(int)x>-0.5)) return((int)ceil(x));
	 else return((int)floor(x));
}

/* Rauschgenerator mit simulierter Normalverteilung */
/* Zur Initialisierung unbedingt RANDOMIZE() aufrufen! */
double noise(double mu,double sigma)
{  double x;
	 /* Zufaelligen Wahrscheinlichkeitswert 0.0<p<=1.0 waehlen */
	 double p = (double)random_nr();
	 /* Normiertes X fuer N(0,1)-Verteilung bei zufaelligem p-Wert berechnen */
	 x = sqrt(-2*log(p));
	 /* Verteilung symmetrisch -> zufaelliger Vorzeichenwechsel */
	 if (random(2)) x = -x;
	 /* X denormiert zurueckgeben */
	 return(sigma * x + mu);
}
void add_noise(pINT l,double stdabw)
{ register int i;
	 for (L_ALL(l,i))
			X(l,i)+=(real_t)noise(0.0,stdabw);
   border_x(l);
}

/* shift horizontal */
void shift_h(pINT l,int delta)
{ pINT x1,y1;
   pINT upr=L_Tab[l].upr;
   pINT xstart=(delta>=0)?L_Tab[l].upr-1:0;
   pINT xend=(delta>=0)?-1:L_Tab[l].upr;
   for (x1=xstart;x1!=xend+delta;x1-=delta)
      for (y1=0;y1<L_Tab[l].size/upr;y1++)
         {X(l,y1*upr+x1)=X(l,y1*upr+x1-delta);
         if (x1-delta==xend+delta) X(l,y1*upr+x1-delta)=0.0;}
}

/* shift horizontal zyklisch for delta = +/-1 */
void shift_hz(pINT l,int delta)
{ pINT x1,y1;
	REAL x2;
  pINT upr=L_Tab[l].upr;
  pINT xstart=(delta>=0)?L_Tab[l].upr-1:0;
  pINT xend=(delta>=0)?-1:L_Tab[l].upr;
  for (y1=0;y1<L_Tab[l].size/upr;y1++)
    {x2=X(l,y1*upr+xstart);
	  for (x1=xstart;x1!=xend+delta;x1-=delta)
			X(l,y1*upr+x1)=X(l,y1*upr+x1-delta);
    X(l,y1*upr+xend+delta)=x2;
		}
}

void shift_v(pINT l,int delta)
{ register int x1,y1;
  pINT upr=L_Tab[l].upr;
  pINT size=L_Tab[l].size;
  pINT ystart=(delta>=0)?size/upr-1:0;
  pINT yend=(delta>=0)?-1:size/upr;
  for (y1=ystart;y1!=yend+delta;y1-=delta)
    for (x1=0;x1<upr;x1++)
      {X(l,y1*upr+x1)=X(l,(y1-delta)*upr+x1);
       if (y1-delta==yend+delta) X(l,(y1-delta)*upr+x1)=0.0;}
}

void mirror_x(pINT l)
{ pINT x1,y1;
  REAL val;
  pINT ymax=L_Tab[l].size/L_Tab[l].upr;
  pINT upr=L_Tab[l].upr;
  for (y1=0;y1<ymax/2;y1++)
    for (x1=0;x1<L_Tab[l].upr;x1++)
      {val=X(l,y1*upr+x1);
       X(l,y1*upr+x1)=X(l,(ymax-y1-1)*upr+x1);
       X(l,(ymax-y1-1)*upr+x1)=val;}
}
void mirror_y(pINT l)
{ pINT x1,y1;
  REAL val;
  pINT xmax=L_Tab[l].upr;
  pINT upr=L_Tab[l].upr;
  for (x1=0;x1<xmax/2;x1++)
    for (y1=0;y1<L_Tab[l].size/L_Tab[l].upr;y1++)
      {val=X(l,y1*upr+x1);
       X(l,y1*upr+x1)=X(l,y1*upr+xmax-1-x1);
       X(l,y1*upr+xmax-1-x1)=val;}
}
/* schnelle Rotation um Winkel 90 in Grad im Uhrzeigersinn */
void rotate90(pINT l)
{ pINT x1,y1,upr=L_Tab[l].upr,size=L_Tab[l].size;
  for (x1=0;x1<upr;x1++)
    for (y1=0;y1<size/upr;y1++)
      XNEW(l,x1*upr+upr-1-y1)=X(l,y1*upr+x1);
  copy_layer(l);
}

/* Rotation um Winkel a in Grad .. not yet finished !! */
void rotate(pINT l,double a)
{  pINT x1,y1,x2,y2,x3,y3,upr=L_Tab[l].upr,size=L_Tab[l].size,
                      xm=2*upr,ym=2*size/upr;
   double sina=sin((double)a/180.0*3.14159),cosa=cos((double)a/180.0*3.14159);
	 sina=round_0(sina); cosa=round_0(cosa);
	 for(L_ALL(l,x1)) XNEW(l,x1)=0.0;
	 for (x1= -2*upr;x1<2*upr;x1++)
			for (y1= -2*(int)size/upr;y1<2*(int)size/upr;y1++)
				 {x2=round_0( (double)(x1+0.5)*cosa-(double)(y1+0.5)*sina -0.5);
					y2=round_0( (double)(x1+0.5)*sina+(double)(y1+0.5)*cosa -0.5);
					x2=x2+xm-0.5; y2=y2+ym-0.5;
					x2=x2/4; y2=y2/4;
					x3=x1+xm; y3=y1+ym; /* x1 und xm kompensieren sich um 0.5 */
					x3=x3/4;  y3=y3/4;
					XNEW(l,y2*upr+x2)+=0.0625*X(l,y3*upr+x3);}
	 copy_layer(l);
	 border_x(l);
}
